@@ -4,12 +4,6 @@
 // RUN: %clangxx_asan -DWAITPID -O0 %s -o %t && not %run %t 2>&1 | FileCheck %s
 // RUN: %clangxx_asan -DWAITPID -O3 %s -o %t && not %run %t 2>&1 | FileCheck %s
 
-// RUN: %clangxx_asan -DWAIT3 -O0 %s -o %t && not %run %t 2>&1 | FileCheck %s
-// RUN: %clangxx_asan -DWAIT3 -O3 %s -o %t && not %run %t 2>&1 | FileCheck %s
-
-// RUN: %clangxx_asan -DWAIT3_RUSAGE -O0 %s -o %t && not %run %t 2>&1 | FileCheck %s
-// RUN: %clangxx_asan -DWAIT3_RUSAGE -O3 %s -o %t && not %run %t 2>&1 | FileCheck %s
-
 
 #include <assert.h>
 #include <sys/wait.h>
@@ -25,12 +19,6 @@ int main(int argc, char **argv) {
     res = wait(status);
 #elif defined(WAITPID)
     res = waitpid(pid, status, WNOHANG);
-#elif defined(WAIT3)
-    res = wait3(status, WNOHANG, NULL);
-#elif defined(WAIT3_RUSAGE)
-    struct rusage *ru = (struct rusage*)(x + argc * 3);
-    int good_status;
-    res = wait3(&good_status, WNOHANG, ru);
 #endif
     // CHECK: stack-buffer-overflow
     // CHECK: {{WRITE of size .* at 0x.* thread T0}}
