@@ -2,15 +2,19 @@
 // RUN: %run %t 2>&1
 //
 // REQUIRES: stable-runtime
-// XFAIL: powerpc64
 
 // This testcase checks correct interaction between VLAs and allocas.
 
 #include <assert.h>
-#include <alloca.h>
 #include <stdint.h>
+#include <stdlib.h>
 #include "sanitizer/asan_interface.h"
 
+// MSVC provides _alloca instead of alloca.
+#if defined(_MSC_VER) && !defined(alloca)
+# define alloca _alloca
+#endif
+
 #define RZ 32
 
 __attribute__((noinline)) void foo(int len) {
